/**
 *  GraphTFT plugin for the Video Disk Recorder 
 * 
 *  common.c - A plugin for the Video Disk Recorder
 *
 *  (c) 2004-2006 Lars Tegeler, Sascha Volkenandt, Jrg Wendel
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 *
 * $Id: common.c,v 1.2 2006/11/06 10:29:23 root Exp $
 *
 **/

// includes

#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <syslog.h>
#include <sys/stat.h>

#include "common.h"
#include "setup.h"

using std::string;

int logLevel = eloOff;
int logDevice = devSyslog;

//***************************************************************************
// Debug
//***************************************************************************

void d(int eloquence, const char* fmt, ...)
{
   char t[BUFSIZ];
   va_list ap;
   time_t  now;
   FILE* lf = 0;

   va_start(ap, fmt);

   if (logLevel >= eloquence)
   {
      switch (logDevice)
      {
         case devNone: break;

         case devStdOut:
         {
            vsnprintf(t + 21, sizeof t - 21, fmt, ap);
            time(&now);
            strftime(t, sizeof t, "[graphTFT: %H:%M:%S]", localtime(&now));
            t[20] = ' ';
            printf("%s\033[K", t);

            break;
         }

         case devSyslog:
         {
            vsnprintf(t + 10, sizeof t - 10, fmt, ap);
            memcpy(t, "[graphTFT] ", 10);
            syslog(LOG_DEBUG, "%s", t);

            break;
         }

         case devFile:
         {
            if (!lf)
               lf = fopen("/tmp/graphTFT.log", "a");

            if (lf) 
            {
               vsnprintf(t + 20, sizeof t - 20, fmt, ap);
               time(&now);
               strftime(t, sizeof t, "%d.%m.%Y %H:%M:%S", localtime(&now));
               t[19] = ' ';
               fprintf(lf, "%s\n", t);
            }
            
            break;
         }
      }
   }

   va_end(ap);
}

//***************************************************************************
// check if a file exists
//***************************************************************************

bool file_exists(std::string filename) 
{
   struct stat file_stat;

   return (stat(filename.c_str(), &file_stat) == 0) ? true : false;
}

//***************************************************************************
// FIX ME: grrr, why somebody must habe an conversion, 
// some must not, and by some it wont work at all
//***************************************************************************

int isolat1ToUTF8(unsigned char* out, int *outlen, const unsigned char* in, int *inlen)
{
   unsigned char* outstart = out;
   const unsigned char* base = in;
   unsigned char* outend = out + *outlen;
   const unsigned char* inend;
   const unsigned char* instop;
   unsigned int c = *in;

   inend = in + (*inlen);
   instop = inend;
                   
   while (in < inend && out < outend - 1) 
   {
      if (c >= 0x80) 
      {
         *out++= ((c >>  6) & 0x1F) | 0xC0;
         *out++= (c & 0x3F) | 0x80;
         ++in;
         c = *in;
      }

      if (instop - in > outend - out) instop = in + (outend - out); 

      while (c < 0x80 && in < instop) 
      {
         *out++ =  c;
         ++in;
         c = *in;
      }
   }  
 
   if (in < inend && out < outend && c < 0x80) 
   {
      *out++ =  c;
      ++in;
   }

   *outlen = out - outstart;
   *inlen = in - base;
   *out = 0;

   return 0;
}

//***************************************************************************
// Left Trim
//***************************************************************************

char* Str::lTrim(char* buf)
{
   if (buf)
   {
      char *tp = buf;

      while (*tp && strchr("\n\r\t ",*tp)) 
         tp++;

      memmove(buf, tp, strlen(tp) +1);
   }
   
   return buf;
}

//*************************************************************************
// Right Trim
//*************************************************************************

char* Str::rTrim(char* buf)
{
   if (buf)
   {
      char *tp = buf + strlen(buf);

      while (tp >= buf && strchr("\n\r\t ",*tp)) 
         tp--;

      *(tp+1) = 0;
   }
   
   return buf;
}

//*************************************************************************
// All Trim
//*************************************************************************

char* Str::allTrim(char* buf)
{
   return lTrim(rTrim(buf));
}
