//***************************************************************************
// Group VDR/GraphTFT
// File graphtft.h
// Date 31.10.06 - Jrg Wendel
// This code is distributed under the terms and conditions of the
// GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
//--------------------------------------------------------------------------
// Class cPluginGraphTFT
//***************************************************************************

#ifndef __GTFT__HPP__
#define __GTFT__HPP__

#include <vdr/plugin.h>
#include <vdr/config.h>

#include "display.h"
#include "setup.h"
#include "i18n.h"
#include "osd.h"
#include "common.h"

static const char* VERSION        = "0.0.16";
static const char* THEMEVERSION   = "0.1.0";
static const char* DESCRIPTION    = "Display Info on TFT";

static const char* MAINMENUENTRY  = "Graph-TFT";

void *(*fast_memcpy)(void* to, const void* from, size_t len);

class cPluginGraphTFT;


//***************************************************************************
// Class GraphTFTMenuItem
//***************************************************************************

class cGraphTFTMenuItem : public cOsdItem
{
   public:
   
      cGraphTFTMenuItem(const char* aTitle)
         : cOsdItem(aTitle) {}

};

//***************************************************************************
// Class GraphTFTMenu
//***************************************************************************

class cGraphTFTMenu : public cMenuSetupPage
{
   public:

      cGraphTFTMenu(const char* title, cPluginGraphTFT* aPlugin);
      virtual ~cGraphTFTMenu();
      
      virtual eOSState ProcessKey(eKeys key);

  protected:

      void Store() { }

      cPluginGraphTFT* plugin;
      int defMode;

};

//***************************************************************************
// Class cPluginGraphTFT
//***************************************************************************

class cPluginGraphTFT : public cPlugin, cThemeService
{
   private:

      cGraphTFTDisplay* display;
      char* _dev;
      int port;

   public:

      cPluginGraphTFT();
      ~cPluginGraphTFT();

      const char* Version()          { return VERSION; }
      const char* Description()      { return tr(DESCRIPTION); }
      const char* CommandLineHelp();

      bool ProcessArgs(int argc, char* argv[]);
      bool Initialize();
      bool Start();
      void Housekeeping() {}

      const char* MainMenuEntry()    { return GraphTFTSetup.HideMainMenu ? 0 : tr(MAINMENUENTRY); }
      cOsdObject* MainMenuAction();

      cMenuSetupPage* SetupMenu();
      bool SetupParse(const char* Name, const char* Value);
      cGraphTFTDisplay* getDisplay() { return display; }

      bool Service(const char* Id, void* Data);
      int loadThemes();
      void setDefaultMode(int defMode) { display->setDefaultMode(defMode); }
};

//***************************************************************************
#endif  // __GTFT__HPP__
