/**
 *  GraphTFT plugin for the Video Disk Recorder 
 * 
 *  imlibrenderer.c - A plugin for the Video Disk Recorder
 *
 *  (c) 2004 Lars Tegeler, Sascha Volkenandt  
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 *
 * $Id: imlibrenderer.c,v 1.2 2006/11/06 10:30:18 root Exp $
 *
 **/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <locale.h>

#include <string>
#include <vector>
#include <sstream>

#include <string>
#include <utility>
#include <iostream>

#include "imlibrenderer.h"
#include "theme.h"
#include "common.h"

using std::string;
using std::cout;
using std::endl;

//***************************************************************************
// Class ImlibRenderer
//***************************************************************************

ImlibRenderer::ImlibRenderer(int x, int y, int width, int height, 
                             string cfgPath, int utf, string thmPath)
   : Renderer(x, y, width, height, cfgPath, utf, thmPath)
{
   Imlib_Context ctx = imlib_context_new();
   imlib_context_push(ctx);

   // cache

   imlib_set_cache_size(4096 * 1024);
   imlib_set_font_cache_size(2048 * 1024);
 
   // font

   std::ostringstream path; path <<  confPath.c_str() << "/graphTFT/fonts/";
   imlib_add_path_to_font_path(path.str().c_str());
  
   // new image

   _cur_image = new_imlib_image();
   imlib_context_set_image(_cur_image);
   _render_image = new_imlib_image();

	imlib_set_color_usage(256);
}

ImlibRenderer::~ImlibRenderer()
{
	;
}

//---------------------------------------------------------------------------------

Imlib_Image ImlibRenderer::new_imlib_image()
{
   return imlib_create_image(dspWidth, dspHeight);
}

void ImlibRenderer::set_context_to_buffer(Imlib_Image img)
{
	imlib_context_set_image(img);
}

void ImlibRenderer::refresh()
{
	// free old render image

	set_context_to_buffer(_render_image);

   imlib_blend_image_onto_image(_cur_image, 0, 0, 0, dspWidth, dspHeight,
                                dspX + dspWidthBorder, dspY + dspHeightBorder,
                                dspWidth - dspX - (2 * dspWidthBorder), 
                                dspHeight - dspY - (2 * dspHeightBorder));
	clear();
}

void ImlibRenderer::clear()
{
	// clear the current image

	set_context_to_buffer(_cur_image);
	imlib_free_image();
	_cur_image = new_imlib_image();
	set_context_to_buffer(_cur_image);
}


//----------------------------------------------------------------------------------

void ImlibRenderer::image(const char* fname, int x, int y, int coverwidth, int coverheight, bool fullpath, bool fit)
{
   Imlib_Image new_image;
   int width = 0;
   int height = 0;
   std::ostringstream path;

   if (fullpath)
      path << fname;
   else
      path << confPath << "/graphTFT/themes/" << themePath << "/" << fname; 

   // #TODO for speed insert a check if file exists 
   // before load

   new_image = imlib_load_image(path.str().c_str());

   /*
     note, the imlib image context isn't really restored after 
     calling this method, it's just set to whatever 'image'
     parameter is
   */

   if (!new_image)
   { 
      d(1, "The image '%s' could not be loaded", path.str().c_str());

      return ;

//       if (fullpath) 
//       {
//          new_image = imlib_create_cropped_scaled_image(0, 0, 
//                                                        imlib_image_get_width(), 
//                                                        imlib_image_get_height(), 
//                                                        coverwidth, coverheight);
//       }
//       else 
//       {
//          if (!x && !y) 
//             new_image = imlib_create_image(dspWidth, dspHeight);
//          else 
//             new_image = imlib_create_image(1, 1);
//       }

//       imlib_context_set_image(new_image);
//       imlib_image_clear_color(0, 0, 0, (!x && !y) ? 255 : 0);
   }

   imlib_context_set_image(new_image);

   width = imlib_image_get_width();
   height = imlib_image_get_height();

   imlib_context_set_image(_cur_image);

   if (fullpath || fit)
      imlib_blend_image_onto_image(new_image, 0, 0, 0, width, height, x, y, coverwidth, coverheight);
   else
      imlib_blend_image_onto_image(new_image, 0, 0, 0, width, height, x, y, width, height);
  
   imlib_context_set_image(new_image);
   imlib_free_image();
   imlib_context_set_image(_cur_image);
}

int ImlibRenderer::charWidthOf(const char* fontName, int fontSize)
{
   Imlib_Font font;
   int width = 20;
   int height = 20;

   if (!fontName)
   {
      imlib_get_text_size("We need here something like a representive sentence WITH SOME UPPER CASE LETTERS", &width, &height);
      return width / 80;
   }

   std::ostringstream font_name_size;
	font_name_size <<  fontName << "/" << fontSize;

   font = imlib_load_font((font_name_size.str()).c_str());

   if (font)
   {
      imlib_context_set_font(font);
		imlib_context_set_image(_cur_image);
      imlib_get_text_size("We need here something like a representive sentence WITH SOME UPPER CASE LETTERS", &width, &height);
      imlib_free_font();

      width /= 80;
   }  
	else
		d(1, "The font '%s' could not be loaded.", fontName);

   return width;
}

//***************************************************************************
// Render Text
//***************************************************************************

int ImlibRenderer::text(const char* _text, const char* font_name, int size, 
                        int align, int x, int y, int r, int g, int b, 
                        int width, int height, int lines, int dots)
{
   int lineHeight, textWidth, dummy;
   int blankWidth;
   int dotsWidth;
   int currentWidth;
   Imlib_Font font;
   const char* token;
   char* text;
   char* line;

   if (!strlen(_text))
      return 0;

   // load font

   std::ostringstream font_name_size;
   font_name_size <<  font_name << "/" << size;

   if (!(font = imlib_load_font((font_name_size.str()).c_str())))
   {
      d(1, "The font '%s' could not be loaded.", font_name);
      return 0;
   }

   text = new char[2 * strlen(_text) + 1];
   line = new char[2 * strlen(_text) + 1];
   strcpy(text, _text);

   // convert to UTF8

   if (utf8)
   {
      int inLen = strlen(_text);
      int outLen = 2 * inLen;

      isolat1ToUTF8((unsigned char*)text , &outLen, (unsigned char*)_text, &inLen);
   }

   if (width <= 0)
      width = dspWidth;

   imlib_context_set_font(font);
   imlib_context_set_image(_cur_image);
   imlib_context_set_color(r, g, b, 255);

   imlib_get_text_size(text, &textWidth, &lineHeight);
   imlib_get_text_size(" ", &blankWidth, &dummy);
   imlib_get_text_size("...", &dotsWidth, &dummy);

   if (!lines)
      lines = height / lineHeight;

   token = strtok(text, " ");

   for (int l = 1; token && l <= lines; l++)
   {
      int yPos = y + lineHeight * (l-1);

      currentWidth = 0;
      *line = 0;

      for (int w = 0; token; w++) 
      {
         int tokenWidth;

         imlib_get_text_size(token, &tokenWidth, &dummy);
         
         if (currentWidth + tokenWidth + blankWidth > width)
         {
            // passt nicht mehr

            if (!w || l == lines)
            {
               // alleinstehendes Wort oder letzte Zeile,
               // daher abgeschnitten anzeigen
               
               int i = 0;
               char* p = line + strlen(line);

               if (strlen(line)) *p++ = ' ';
               *p = 0;

               while (currentWidth + (dots ? dotsWidth : 0) < width && token[i])
               {
                  *p++ = token[i++];
                  *p = 0;
                  imlib_get_text_size(line, &currentWidth, &dummy);
               }

               if (dots)
               {
                  if (currentWidth + dotsWidth > width)
                     *(p-1) = 0;
                  
                  strcat(line, "...");
               }

               token = strtok(0, " ");
            }

            break;
         }
         else
         {
            // passt noch rein

            currentWidth += blankWidth + tokenWidth;
            sprintf(line, "%s%s%s", line, w ? " " : "", token);

            token = strtok(0, " ");
         }
      }

      if (align != 0)
         imlib_get_text_size(line, &textWidth, &dummy);

      if (align == 0)                // left
         imlib_text_draw(x, yPos, line);
      else if (align == 1)           // center
         imlib_text_draw(x + (width - textWidth) / 2, yPos, line);
      else                           // right
         imlib_text_draw(x + width - textWidth, yPos, line);
   }

   imlib_free_font();
   delete[] text;
   delete[] line;

   return 0;
}


//***************************************************************************
// Draw Text
//***************************************************************************
/*
int ImlibRenderer::text(const char* text, const char* font_name, int size, 
                        int align, int x, int y, int r, int g, int b, 
                        int width, int height, int lines, int dots)
{
	string tmp2;
	string tmp = text;
	int lineHeight, textWidth;
  	Imlib_Font font;
   unsigned int pos;

   if (!strlen(text))
      return 0;

	if (utf8)
   {
      const char out[1024] = "";
      string in = tmp;
      int l1 = in.length();
      int l2 = 1024;

      if (!isolat1ToUTF8((unsigned char*)out , &l2, (unsigned char*)in.c_str(), &l1))
      {
			tmp = strdup(out);
			tmp.resize(l2);
		}
	}

   // load font

   std::ostringstream font_name_size;
	font_name_size <<  font_name << "/" << size;
   font = imlib_load_font((font_name_size.str()).c_str());

   if (width <= 0)
      width = dspWidth;          // ;)

   if (font)
   {
		imlib_context_set_font(font);
		imlib_context_set_image(_cur_image);
		imlib_context_set_color(r, g, b, 255);
		imlib_get_text_size(tmp.c_str(), &textWidth, &lineHeight);

      unsigned int charPerLine = width / charWidthOf();
      
		if (!lines)
			lines = height / lineHeight;

      pos = 0;

      for (int l = 0; l < lines && pos < tmp.length(); l++)
      {
         int yPos = y + lineHeight * l;
         string next;
         tmp2 = "";

         do
         {
            next = tmp.substr(pos, tmp.find(" ", pos) - pos + (next == "" ? 0 :  1));

            // #TODO superbuggy !!!

            if (next == "") break;    // we are ready !
            
            if (tmp2 == "" && next.length() >= charPerLine)
            {
               // this single word don't fit in line :(

               tmp2 = tmp2 + next;
               pos += next.length();

               break;
            }               

            if (tmp2.length() + next.length() >= charPerLine)
            {
               // line full

               if (l == lines-1 && dots)
                  tmp2 = tmp2 + next.substr(0, next.length()-1) + "..";  // -1 enough for the 2 dots ;)
               
               break;
            }
               
            tmp2 = tmp2 + next;
            pos += next.length();


         } while (pos < tmp.length());

         if (tmp2 == "")
            break;

         if (align != 0)
            imlib_get_text_size(tmp2.c_str(), &textWidth, &lineHeight);

         if (align == 0)                // left
				imlib_text_draw(x, yPos, tmp2.c_str());
			else if (align == 1)           // center
				imlib_text_draw(x + (width - textWidth) / 2, yPos, tmp2.c_str());
			else                           // right
				imlib_text_draw(x + width - textWidth, yPos, tmp2.c_str());
      }


      imlib_free_font();
	}

	else
		d(1, "The font '%s' could not be loaded.", font_name);

	return 0;
}
*/

//***************************************************************************
// Draw Rectangle
//***************************************************************************

void ImlibRenderer::rectangle(int x, int y, 
                              int width, int height, 
                              int r, int g, int b, 
                              int tr) 
{
	imlib_context_set_image(_cur_image);
	imlib_context_set_color(r, g, b, tr);
	imlib_image_fill_rectangle(x, y, width, height);
}

//--------------------------------------------------------------------------------------

void ImlibRenderer::dumpImage2File(const char* fname, int dumpWidth, 
                                   int dumpHeight, const char* aPath)
{
	std::ostringstream path; 
   const char* format;

   if (aPath && *aPath)
      path << aPath;
   else
      path << "/tmp";
   
   format = strchr(fname, '.') + 1;

	// check if local directory structure exist, else create it.

  	if (!file_exists(path.str().c_str())) 
      mkdir(path.str().c_str(), 0777);

	/* an image handle */

  	Imlib_Image new_image = imlib_create_image(dumpWidth, dumpHeight);

	/* get width and heigt */

	imlib_context_set_image(_render_image);
	int width = imlib_image_get_width();
	int height = imlib_image_get_height();

	imlib_context_set_image(new_image);
	imlib_blend_image_onto_image(_render_image, 0, 0, 0, width, height, 
                                0, 0, dumpWidth, dumpHeight);

	/* save the image */

	imlib_image_set_format(format && *format ? format : "png");
	path << "/" << fname;
	imlib_save_image(path.str().c_str());

	imlib_free_image();
	imlib_context_set_image(_render_image);
}

//***************************************************************************
// Get Image Buffer
//***************************************************************************

int ImlibRenderer::getImageBuffer(const void*& buffer, int quality)
{
   int size = 0;

#ifdef HAVE_IMAGE_MAGICK

   int w, h;
   DATA32* p;

   imlib_context_set_image(_render_image);
   
   p = imlib_image_get_data_for_reading_only();
   w = imlib_image_get_width();
   h = imlib_image_get_height();
   
   if (p)
   {
      Image image(w, h, "BGRA", CharPixel, p);
      
      image.magick("JPEG");       // set format
      image.quality(quality);     // set quality
      image.write(&blob);
      
      buffer = blob.data();
      size = blob.length();
   }

#endif

   return size;
}
