/**
 *  GraphTFT plugin for the Video Disk Recorder 
 * 
 *  setup.c - A plugin for the Video Disk Recorder
 *
 *  (c) 2004-2006 Lars Tegeler, Sascha Volkenandt, Jrg Wendel
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 *
 * $Id: setup.c,v 1.2 2006/11/06 10:29:23 root Exp $
 *
 **/

#include "common.h"
#include "setup.h"

#ifdef HAVE_DFB
# include "cache.h"
#endif

//***************************************************************************
// Log Devices
//***************************************************************************

static const char* logDevices[] = 
{
   "none",
   "stdout",
   "syslog",
   "file"
};

cGraphTFTSetup GraphTFTSetup;

//***************************************************************************
// Object
//***************************************************************************

cGraphTFTSetup::cGraphTFTSetup(void)
{
    Theme = "";
    index = 0;
    Level=0;
    LogDevice=0;
    Iso2Utf = 1;
    HideMainMenu = 0;
    ShowMutePermanent = 0;
    UseStillPicture = 1;
    DvbRefresh = 100;
    VdrHeight = 5;
    VdrWidth = 25;
    VdrX=0;
    VdrY=0;
    DumpImage = 0;
    DumpRefresh = 5;
    DumpImageX = 720;
    DumpImageY = 576;
    OSDOffsetX = 50;
    OSDOffsetY = 400;
    OSDMessageTime = 30;
    PluginConfPath = strdup("/tmp");
    CoverImage = 0;
    ViewType = 0;
    ViewLength = 0;
    ScrollTime = 50;
    ScrollCount = 0;
    JpegQuality = 60;
    enableSpectrumAnalyzer = 0;
}

//***************************************************************************
// Setup Parse
//***************************************************************************

bool cGraphTFTSetup::SetupParse(const char* Name, const char* Value)
{
    // Take Cover Images

    if (!strcasecmp(Name, "CoverImage")) 
    {
       if (CoverImage) 
          free(CoverImage);

       CoverImage = strdup(Value);
    }

    else if (!strcasecmp(Name, "Theme"))  Theme = Value;
    else if (!strcasecmp(Name, "HideMainMenu"))  HideMainMenu  = atoi(Value);
    else if (!strcasecmp(Name, "ShowMutePermanent"))  ShowMutePermanent = atoi(Value);
    else if (!strcasecmp(Name, "Iso2Utf"))  Iso2Utf  = atoi(Value);
    else if (!strcasecmp(Name, "UseStillPicture"))  UseStillPicture  = atoi(Value);
    else if (!strcasecmp(Name, "SpectrumAnalyzer"))  enableSpectrumAnalyzer  = atoi(Value);
    else if (!strcasecmp(Name, "DvbRefresh"))  DvbRefresh  = atoi(Value);
    else if (!strcasecmp(Name, "VdrHeight"))  VdrHeight  = atoi(Value);
    else if (!strcasecmp(Name, "VdrWidht"))  VdrWidth  = atoi(Value);
    else if (!strcasecmp(Name, "VdrX"))  VdrX  = atoi(Value);
    else if (!strcasecmp(Name, "VdrY"))  VdrY  = atoi(Value);
    else if (!strcasecmp(Name, "DumpImage"))  DumpImage  = atoi(Value);
    else if (!strcasecmp(Name, "DumpRefresh"))  DumpRefresh  = atoi(Value);
    else if (!strcasecmp(Name, "DumpImageX"))  DumpImageX  = atoi(Value);
    else if (!strcasecmp(Name, "DumpImageY"))  DumpImageY  = atoi(Value);
    else if (!strcasecmp(Name, "OSDOffsetX"))  OSDOffsetX  = atoi(Value);
    else if (!strcasecmp(Name, "OSDOffsetY"))  OSDOffsetY  = atoi(Value);
    else if (!strcasecmp(Name, "OSDMessageTime"))  OSDMessageTime  = atoi(Value);
    else if (!strcasecmp(Name, "LogDevice"))  LogDevice  = atoi(Value);
    else if (!strcasecmp(Name, "Level"))  Level  = atoi(Value);
    else if (!strcasecmp(Name, "ViewType"))  ViewType  = atoi(Value);
    else if (!strcasecmp(Name, "ViewLength"))  ViewLength  = atoi(Value);
    else if (!strcasecmp(Name, "ScrollTime"))  ScrollTime  = atoi(Value);
    else if (!strcasecmp(Name, "ScrollCount"))  ScrollCount  = atoi(Value);
    else if (!strcasecmp(Name, "JpegQuality"))  JpegQuality  = atoi(Value);

    else return false;

    return true;
}

//***************************************************************************
// cMenuSetupGraphTFT
//***************************************************************************

cMenuSetupGraphTFT::cMenuSetupGraphTFT(cGraphTFTDisplay* graphTFTdisplay)
{
   cGraphTFTTheme* t;
   char* buf = 0;
   int i;

   for (int i = 0; i < 100; i++)
      themeNames[i] = 0;

   _data = GraphTFTSetup;
   _graphTFTdisplay = graphTFTdisplay;

   SetSection(tr("GraphTFT"));

   _data.index = theTheme->Index();

   for (i = 0, t = themes.First(); t && i < 100; t = themes.Next(t), i++)
      asprintf(&themeNames[i], t->getName().c_str());

   if (themes.Count())
      Add(new cMenuEditStraItem(tr("Theme"), &_data.index, themes.Count(), themeNames));

   Add(new cMenuEditBoolItem(tr("Hide Mainmenu Entry"),        &_data.HideMainMenu));
   Add(new cMenuEditBoolItem(tr("Show Mute Symbol permanent"), &_data.ShowMutePermanent));
   Add(new cMenuEditIntItem(tr("Title view type"),             &_data.ViewType,0,3));
   Add(new cMenuEditIntItem(tr("Title view length"),           &_data.ViewLength,10,50));
   Add(new cMenuEditIntItem(tr("Scroll Time (ms)"),            &_data.ScrollTime,10,1000));
   Add(new cMenuEditIntItem(tr("Scroll Count"),                &_data.ScrollCount,0,1000));
   Add(new cMenuEditBoolItem(tr("Dump image to file"),         &_data.DumpImage));
   Add(new cMenuEditIntItem(tr("Dump Refresh"),                &_data.DumpRefresh,0,600));
   Add(new cMenuEditIntItem(tr("Dump image width"),            &_data.DumpImageX,1,1024));
   Add(new cMenuEditIntItem(tr("Dump image height"),           &_data.DumpImageY,1,768));
   Add(new cMenuEditBoolItem(tr("Convert Iso to UTF-8"),       &_data.Iso2Utf));
   Add(new cMenuEditIntItem(tr("X Offset for OSD"),            &_data.OSDOffsetX,0,719));
   Add(new cMenuEditIntItem(tr("Y Offset for OSD"),            &_data.OSDOffsetY,0,575));
   Add(new cMenuEditIntItem(tr("Timeout for OSD Messages"),    &_data.OSDMessageTime,0,3600));
   Add(new cMenuEditBoolItem(tr("Spectrum Analyzer"),          &_data.enableSpectrumAnalyzer));

#ifdef HAVE_IMLIB
   asprintf(&buf, "--- %s --------------------------------------------", tr("DVB Device"));
   Add(new cOsdItem(buf));
   free(buf);
   Add(new cMenuEditBoolItem(tr("Use StillPicture"),           &_data.UseStillPicture));
   Add(new cMenuEditIntItem(tr("Refresh"),                     &_data.DvbRefresh,0,1024));
   asprintf(&buf, "--- %s --------------------------------------------", tr("DVB/FB Device"));
   Add(new cOsdItem(buf));
   free(buf);
   Add(new cMenuEditIntItem(tr("X Offset"),                    &_data.VdrX,0,719));
   Add(new cMenuEditIntItem(tr("Y Offset"),                    &_data.VdrY,0,575));
   Add(new cMenuEditIntItem(tr("Border to Height"),            &_data.VdrHeight,0,719));
   Add(new cMenuEditIntItem(tr("Border to Width"),             &_data.VdrWidth,0,576));
   asprintf(&buf, "--- %s -----------------------------------------", tr("TCP Connection"));
   Add(new cOsdItem(buf));
   free(buf);
   Add(new cMenuEditIntItem(tr("Jpeg Quality"),                &_data.JpegQuality, 0, 100));

#endif

#ifdef HAVE_DFB
   asprintf(&buf, "--- %s --------------------------------------------", tr("directFB Device"));
   Add(new cOsdItem(buf));
   free(buf);
#endif

   asprintf(&buf, "--- %s --------------------------------------------", tr("Debug"));
   Add(new cOsdItem(buf));
   free(buf);
   Add(new cMenuEditStraItem(tr("Log Device"),                 &_data.LogDevice, 4, logDevices));
   Add(new cMenuEditIntItem(tr("Debug Level"),                 &_data.Level,0,10));

   SetHelp(0,0,0,0);
}

//***************************************************************************
// 
//***************************************************************************

cMenuSetupGraphTFT::~cMenuSetupGraphTFT()
{
   for (int i = 0; themeNames[i] && i < 100; i++)
      free(themeNames[i]);
}

//***************************************************************************
// 
//***************************************************************************

eOSState cMenuSetupGraphTFT::ProcessKey(eKeys Key)
{
   d(4, "cMenuSetupGraphTFT::ProcessKey()");

   eOSState state = cOsdMenu::ProcessKey(Key);

   switch (Key)
   {
      case kRed:    state = osUser1;  break;
      case kGreen:  state = osUser2;  break;
      case kYellow: state = osUser3;  break;
      case kBlue:   state = osUser4;  break;
      default:      break;
   }

   switch (state)
   {
      case osUnknown:

         if (Key == kOk)
         {
#ifdef HAVE_DFB   
            // clear image cache

            if (GraphTFTSetup.Theme != _data.Theme)
               imageCache.Clear();
#endif
            Store();
            state = osBack;
         }
         break;

      case osUser1:   //Red

         //start/stop update thread

         if ( _graphTFTdisplay->Active())
            _graphTFTdisplay->Stop();
         else
            _graphTFTdisplay->Start();

         state = osEnd;

         break;

      case osUser2:   //Green
         state = osContinue;
         break;

      case osUser3:   //Yellow
         state = osContinue;
         break;

      case osUser4:   //Blue
         state = osContinue;
         break;

      case osBack:    //Back
         break;

      default:
         break;
   }

   return state;
}

//***************************************************************************
// Store
//***************************************************************************

void cMenuSetupGraphTFT::Store(void)
{
    GraphTFTSetup = _data;

    theTheme = themes.Get(GraphTFTSetup.index);
    GraphTFTSetup.Theme = theTheme->getName();

    SetupStore("Theme", GraphTFTSetup.Theme.c_str());
    SetupStore("HideMainMenu", GraphTFTSetup.HideMainMenu);
    SetupStore("ShowMutePermanent", GraphTFTSetup.ShowMutePermanent);
    SetupStore("ViewType", GraphTFTSetup.ViewType);
    SetupStore("ViewLength", GraphTFTSetup.ViewLength);
    SetupStore("ScrollTime", GraphTFTSetup.ScrollTime);
    SetupStore("ScrollCount", GraphTFTSetup.ScrollCount);
    SetupStore("Iso2Utf", GraphTFTSetup.Iso2Utf);
    SetupStore("UseStillPicture", GraphTFTSetup.UseStillPicture);
    SetupStore("SpectrumAnalyzer", GraphTFTSetup.enableSpectrumAnalyzer);
    SetupStore("DvbRefresh", GraphTFTSetup.DvbRefresh);
    SetupStore("VdrHeight", GraphTFTSetup.VdrHeight);
    SetupStore("VdrWidht", GraphTFTSetup.VdrWidth);
    SetupStore("VdrX", GraphTFTSetup.VdrX);
    SetupStore("VdrY", GraphTFTSetup.VdrY);
    SetupStore("DumpImage", GraphTFTSetup.DumpImage);
    SetupStore("DumpRefresh", GraphTFTSetup.DumpRefresh);
    SetupStore("DumpImageX", GraphTFTSetup.DumpImageX);
    SetupStore("DumpImageY", GraphTFTSetup.DumpImageY);
    SetupStore("OSDOffsetX", GraphTFTSetup.OSDOffsetX);
    SetupStore("OSDOffsetY", GraphTFTSetup.OSDOffsetY);
    SetupStore("OSDMessageTime", GraphTFTSetup.OSDMessageTime);
    SetupStore("LogDevice", GraphTFTSetup.LogDevice);
    SetupStore("Level", GraphTFTSetup.Level);
    SetupStore("JpegQuality", GraphTFTSetup.JpegQuality);

    if (_graphTFTdisplay)    
       _graphTFTdisplay->setupChanged();
}
