/**
 *  GraphTFT plugin for the Video Disk Recorder 
 * 
 *  theme.c - A plugin for the Video Disk Recorder
 *
 *  (c) 2004 Lars Tegeler, Sascha Volkenandt  
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 *
 * $Id: theme.c,v 1.2 2006/11/06 10:29:23 root Exp $
 *
 **/

#include "theme.h"
#include "common.h"

//***************************************************************************
// The Theme
//***************************************************************************

cGraphTFTTheme* theTheme = 0;
cGraphTFTThemes themes;

//***************************************************************************
// cThemeService
//***************************************************************************

const char* cThemeService::sections[] =
{
   "Theme",

   "NormalTV",
   "NormalRadio",
   "NormalClock",
   "NormalProviderLogo",
   "NormalSysinfo",
   "NormalSoccer",

   "ReplayNormal",
   "ReplayDVD",
   "ReplayMP3",

   "MenuCommon",
   "Menu",
   "MenuWhatsOnElse",
   "MenuWhatsOn",
   "MenuWhatsOnNow",
   "MenuWhatsOnNext",
   "MenuMain",
   "MenuSchedule",
   "MenuChannels",
   "MenuTimers",
   "MenuRecordings",
   "MenuSetup",
   "MenuSetupPage",
   "MenuCommands",

   "Calibration",

   0
};

const char* cThemeService::toName(eThemeSection aSection)
{
   if (!isValid(aSection))
      return "unknown";

   return sections[aSection];
}

cThemeService::eThemeSection cThemeService::toSection(const char* aName)
{
   for (int i = 0; sections[i]; i++ )
      if (strcasecmp(sections[i], aName) == 0)
         return (eThemeSection)i;

   return sectionUnknown;
}

//***************************************************************************
// Class cGraphTFTThemeItem
//***************************************************************************

cGraphTFTThemeItem::cGraphTFTThemeItem(void)
{
   _item = itemNone;
   _section = 0;
   _x=1;
   _y=1;
   _bg_x=0;
   _bg_y=0;
   _image_map = 0;
   _stat_pic=0;
   _stat_text=0;
   _stat_width = 0;
   _stat_x=0;
   _stat_y=0;
   _width=0;
   _height=0;
   _bg_width=0;
   _bg_heigth=0;
   _red=255;
   _green=255;
   _blue=255;
   _transparent=255;
   _bg_red=0;
   _bg_green=0;
   _bg_blue=0;
   _bg_transparent=255;
   _lines=0;
   _size=32;
   _sw=0;
   _align=0;
   _animated=0;
   _delay=0;
   _count=0;
   _spacing = 0;
   _bar_height = 0;
   _bar_height_unit = iuAbsolute;

   _font = "Vera";
   _type = "";
   _format = "png";
   _path = "";
   _path2 = "";
   _text = "";
   _number = 0;
   _index = 0;
   _onClick = "";
   _onDblClick = "";
   _sectionInclude = sectionUnknown;
}

cGraphTFTThemeItem::cGraphTFTThemeItem(cGraphTFTThemeItem* item)
{
   _item = item->_item;
   _section = item->_section;
   _x = item->_x;
   _y = item->_y;
   _bg_x = item->_bg_x;
   _bg_y = item->_bg_y;
   _image_map = item->_image_map;
   _stat_pic = item->_stat_pic;
   _stat_text = item->_stat_text;
   _stat_width = item->_stat_width;
   _stat_x = item->_stat_x;
   _stat_y = item->_stat_x;
   _width = item->_width;
   _height = item->_height;
   _bg_width = item->_bg_width;
   _bg_heigth = item->_bg_heigth;
   _red = item->_red;
   _green = item->_green;
   _blue = item->_blue;
   _transparent = item->_transparent;
   _bg_red = item->_bg_red;
   _bg_green = item->_bg_green;
   _bg_blue = item->_bg_blue;
   _bg_transparent = item->_bg_transparent;
   _lines = item->_lines;
   _size = item->_size;
   _sw = item->_sw;
   _align = item->_align;
   _animated = item->_animated;
   _delay = item->_delay;
   _count = item->_count;
   _spacing = item->_spacing;
   _bar_height = item->_bar_height;
   _bar_height_unit = item->_bar_height_unit;
   _font = item->_font;
   _type = item->_type;
   _format = item->_format;
   _path = item->_path;
   _path2 = item->_path2;
   _text = item->_text;
   _number = item->_number;
   _index = item->_index;
   _onClick = item->_onClick;
   _onDblClick = item->_onDblClick;
   _sectionInclude = item->_sectionInclude;
}

cGraphTFTThemeItem::~cGraphTFTThemeItem()
{
   // nothing yet ..
}

//***************************************************************************
// Parse one line => one Item
//***************************************************************************

bool cGraphTFTThemeItem::Parse(const char* s)
{
   static cThemeSection* currentSection = 0;

   int theSection;
   string tmp;
   int posA = -1;
   int posB = -1;
   std::string toParse = strdup(s); 

   // Skip comments
   // #TODO detect comment even with spaces before '#' !

   if (toParse[0] == '#')
      return true;

   // check if it is a section start

   posA = toParse.find("[");
   posB = toParse.find("]", posA);

   if (posA == 0 && posB == ((int)toParse.size() - 1))
   {
      tmp = toParse.substr(posA + 1, posB-1);

      // what section is it?

      if ((theSection = cThemeService::toSection(tmp.c_str())) == sectionUnknown)
      {
         isyslog("Warning: Skipping unknown section '%s'", tmp.c_str());
         currentSection = 0;

         return true;
      }

      // dereference aliases ..

      if (theSection == sectionMenuWhatsOn)
         theSection = sectionMenuWhatsOnElse;

      // append section

      currentSection = new cThemeSection(theSection);
      theTheme->getSections()->Add(currentSection);
   }

   else if (currentSection)
   {
      // no section, so it must be an item

      if (ParseVar(toParse, "Include", &tmp))
      {
         if ((_sectionInclude = cThemeService::toSection(tmp.c_str())) == sectionUnknown)
         {
            isyslog("Warning: Ignoring include of unknown section '%s'", tmp.c_str());
            return true;
         }

         _section = currentSection->getId();      
         _item = itemSectionInclude;

         return true;
      }

      if (!ParseVar(toParse, "Item", &tmp))
         return false;

      _section = currentSection->getId();

      // Look which item it is

      if (tmp == "Date")
      {
         _item = itemDate;
         ParseText(toParse);
         
      }else if (tmp == "Column")
      {
         _item = itemColumn;
         _x = -1;
         ParseText(toParse);

      }else if (tmp == "ColumnSelected")
      {
         _item = itemColumnSelected;
         _x = -1;
         ParseText(toParse);

      }else if (tmp == "PartingLine")
      {
         _item = itemPartingLine;
         _x = -1;
         ParseText(toParse);

      }else if (tmp == "DateOnly")
      {
         _item = itemDateOnly;
         ParseText(toParse);

      }else if (tmp == "TimeOnly")
      {
         _item = itemTimeOnly;
         ParseText(toParse);

      }else if (tmp == "Channel")
      {
         _item = itemChannel;
         ParseText(toParse);

      }else if (tmp == "ChannelOnly")
      {
         _item = itemChannelOnly;
         ParseText(toParse);

      }else if (tmp == "ChannelNumber")
      {
         _item = itemChannelNumber;
         ParseText(toParse);

      }else if (tmp == "Rectangle")
      {
         _item = itemRectangle;
         ParseText(toParse);

      }else if (tmp ==  "PresentTitle")
      {
         _item = itemPresentTitle;
         ParseText(toParse);
         
      }else if (tmp == "PresentSubtitle")
      {
         _item = itemPresentSubtitle;
         ParseText(toParse);

      }else if (tmp ==  "PresentTime")
      {
         _item = itemPresentTime;
         ParseText(toParse);
         
      }else if (tmp ==  "FollowingTitle")
      {
         _item = itemFollowingTitle;
         ParseText(toParse);
         
      }else if (tmp == "FollowingSubtitle")
      {
         _item = itemFollowingSubtitle;
         ParseText(toParse);

      }else if (tmp ==  "FollowingTime")
      {
         _item = itemFollowingTime;
         ParseText(toParse);
                     
      }else if (tmp == "Recording")
      {
         _item = itemRecording;
         ParseText(toParse);
         
      }else if (tmp == "Text" )
      {
         _item = itemText;
         ParseText(toParse);

      }else if (tmp == "Message")
      {
         _item=itemMessage;
         ParseText(toParse);

      }else if (tmp == "Volumebar")
      {
         _item=itemVolumebar;
         ParseText(toParse);

      }else if (tmp == "VolumeMuteSymbol")
      {
         _item=itemVolumeMuteSymbol;
         ParseText(toParse);

      }else if (tmp == "VolumeBackground")
      {
         _item=itemVolumeBackground;
         ParseText(toParse);

      }else if (tmp == "MessageBackground")
      {
         _item=itemMessageBackground;
         ParseText(toParse);

      }else if (tmp == "SymRecording")
      {
         _item=itemSymRecording;
         ParseText(toParse);

      }else if (tmp == "Sym2ch")
      {       
         _item=itemSym2ch;
         ParseText(toParse);

      }else if (tmp == "SymDD")
      {       
         _item=itemSymDD;
         ParseText(toParse);
      
      }else if (tmp == "SymVTX")
      {       
         _item=itemSymVTX;
         ParseText(toParse);

      }else if (tmp == "SymCrypt")
      {       
         _item=itemSymCrypt;
         ParseText(toParse);

      }else if (tmp == "Logo")
      {
         _item=itemLogo;
         ParseText(toParse);

      }else if (tmp == "Timebar")
      {
         _item=itemTimebar;
         ParseText(toParse);

      }else if (tmp == "Image")
      {
         _item=itemImage;
         ParseText(toParse);
      }else if (tmp == "ImageCover")
      {
         _item=itemImageCover;
         ParseText(toParse);

      }else if (tmp == "ImageFile")
      {
         _item=itemImageFile;
         ParseText(toParse);
            
      }else if (tmp == "ReplayName")
      {
         _item=itemReplayName;
         ParseText(toParse);

      }else if (tmp == "ReplayFolder")
      {
         _item=itemReplayFolder;
         ParseText(toParse);

      }else if (tmp == "ReplayProcessbar")
      {
         _item=itemReplayProcessbar;
         ParseText(toParse);
      
      }else if (tmp == "ReplayTotal")
      {
         _item=itemReplayTotal;
         ParseText(toParse);
      
      }else if (tmp == "ReplayCurrent")
      {
         _item=itemReplayCurrent;
         ParseText(toParse);
      
      }else if (tmp == "ReplayRemaining")
      {
         _item=itemReplayRemaining;
         ParseText(toParse);
      
      }else if (tmp == "ReplaySymbolPlay")
      {
         _item=itemSymRepPlay;
         ParseText(toParse);

      }else if (tmp == "ReplaySymbolPause")
      {
         _item=itemSymRepPause;
         ParseText(toParse);
      
      }else if (tmp == "ReplaySymbolFF")
      {
         _item=itemSymRepFF;
         ParseText(toParse);
      
      }else if (tmp == "ReplaySymbolFB")
      {
         _item=itemSymRepFB;
         ParseText(toParse);
      
      }else if (tmp == "ReplaySymbolSF")
      {
         _item=itemSymRepSF;
         ParseText(toParse);
      
      }else if (tmp == "ReplaySymbolSB")
      {
         _item=itemSymRepSB;
         ParseText(toParse);
               
      }else if (tmp == "MenuTitle")
      {
         _item=itemMenuTitle;
         ParseText(toParse);

      }else if (tmp == "Menu")
      {
         _item=itemMenu;
         ParseText(toParse);

      }else if (tmp == "MenuSelected")
      {
         _item=itemMenuSelected;
         ParseText(toParse);

      }else if (tmp == "MenuStaticPicture")
      {
         _item=itemMenuStaticPicture;
         ParseText(toParse);

      }else if (tmp == "MenuButtonRed")
      {
         _item=itemMenuButtonRed;
         ParseText(toParse);

      }else if (tmp == "MenuButtonGreen")
      {
         _item=itemMenuButtonGreen;
         ParseText(toParse);

      }else if (tmp == "MenuButtonYellow")
      {
         _item=itemMenuButtonYellow;
         ParseText(toParse);

      }else if (tmp == "MenuButtonBlue")
      {
         _item=itemMenuButtonBlue;
         ParseText(toParse);

      }else if (tmp == "MenuButtonBackgroundRed")
      {
         _item=itemMenuButtonBackgroundRed;
         ParseText(toParse);

      }else if (tmp == "MenuButtonBackgroundGreen")
      {
         _item=itemMenuButtonBackgroundGreen;
         ParseText(toParse);
      
      }else if (tmp == "MenuButtonBackgroundYellow")
      {
         _item=itemMenuButtonBackgroundYellow;
         ParseText(toParse);

      }else if (tmp == "MenuButtonBackgroundBlue")
      {
         _item=itemMenuButtonBackgroundBlue;
         ParseText(toParse);
         
      }else if (tmp == "MenuText")
      {
         _item=itemMenuText;
         ParseText(toParse);

      }else if (tmp == "MenuTextBackground")
      {
         _item=itemMenuTextBackground;
         ParseText(toParse);   
         
      }else if (tmp == "MenuImageMap")
      {
         _item=itemMenuImageMap;
         ParseText(toParse);
         
      }else if (tmp == "DateTimeFormat")
      {
         _item = itemDateTimeFormat;
         ParseText(toParse);

      }else if (tmp == "SpectrumAnalyzer")
      {
         _item = itemSpectrumAnalyzer;
         ParseText(toParse);

      }else if (tmp == "ClickArea")
      {
         _item = itemClickArea;
         ParseText(toParse);

      }else if (tmp == "MenuNavigationArea")
      {
         _item = itemMenuNavigationArea;
         ParseText(toParse);

      }else if (tmp == "CalibrationCursor")
      {
         _item = itemCalibrationCursor;
         ParseText(toParse);

      } else if (tmp == "Theme")
      {
         // or is it the theme item, then get 
         // name, version, dir, ...

         string value;

         _item = itemTheme;

         ParseVar(toParse, "name", &value);
         theTheme->setName(value);
         ParseVar(toParse, "version", &value);
         theTheme->setVersion(value);
         ParseVar(toParse, "dir", &value);
         theTheme->setDir(value);
         ParseVar(toParse, "startImage", &value);
         theTheme->setStartImage(value);
         ParseVar(toParse, "endImage", &value);
         theTheme->setEndImage(value);
      }

      else
      {
         // ups, none item known witch that name

         d(0, "Ignoring unknown theme item = %s\n", tmp.c_str());
      }
   }

   return true;
}

//***************************************************************************
// An item can parse the line for the text values
//***************************************************************************

bool cGraphTFTThemeItem::ParseText(std::string toParse)
{
   ParseVar(toParse,"x",&_x);
   ParseVar(toParse,"y",&_y);
   ParseVar(toParse,"width",&_width);
   ParseVar(toParse,"height",&_height);
   ParseVar(toParse,"red",&_red);
   ParseVar(toParse,"green",&_green);
   ParseVar(toParse,"blue",&_blue);
   ParseVar(toParse,"transperents",&_transparent);         // compatibility ;)
   ParseVar(toParse,"transparent",&_transparent);
   ParseVar(toParse,"lines",&_lines);
   ParseVar(toParse,"font",&_font);
   ParseVar(toParse,"size",&_size);
   ParseVar(toParse,"text",&_text);
   ParseVar(toParse,"switch",&_sw);             // bool
   ParseVar(toParse,"align",&_align);
   ParseVar(toParse,"path",&_path);
   ParseVar(toParse,"pathBACK",&_path);
   ParseVar(toParse,"pathFRONT",&_path2);
   ParseVar(toParse,"pathON",&_path);
   ParseVar(toParse,"pathOFF",&_path2);
   ParseVar(toParse,"name",&_path);
   ParseVar(toParse,"file",&_path2);
   ParseVar(toParse,"path2",&_path2);
   ParseVar(toParse,"type",&_type);
   ParseVar(toParse,"format",&_format);
   ParseVar(toParse,"animated",&_animated);     // bool
   ParseVar(toParse,"delay",&_delay);
   ParseVar(toParse,"count",&_count);
   ParseVar(toParse,"image_map",&_image_map);   // bool
   ParseVar(toParse,"stat_pic",&_stat_pic);     // bool
   ParseVar(toParse,"stat_text",&_stat_text);   // bool
   ParseVar(toParse,"stat_width",&_stat_width);
   ParseVar(toParse,"stat_x",&_stat_x);
   ParseVar(toParse,"stat_y",&_stat_y);
   ParseVar(toParse,"bg_x",&_bg_x);
   ParseVar(toParse,"bg_y",&_bg_y);
   ParseVar(toParse,"bg_width",&_bg_width);
   ParseVar(toParse,"bg_heigth",&_bg_heigth);
   ParseVar(toParse,"bg_red",&_bg_red);
   ParseVar(toParse,"bg_green",&_bg_green);
   ParseVar(toParse,"bg_blue",&_bg_blue);
   ParseVar(toParse,"bg_transperents",&_bg_transparent);   // compatibility ;)
   ParseVar(toParse,"bg_transparent",&_bg_transparent);
   ParseVar(toParse,"number",&_number);
   ParseVar(toParse,"spacing",&_spacing);
   ParseVar(toParse,"on_click",&_onClick);
   ParseVar(toParse,"on_dblclick",&_onDblClick);

   // now some items with special handling

   string tmp;

   if (ParseVar(toParse,"bar_height",&tmp))
   {
      _bar_height = atoi(tmp.c_str());

      if (tmp.find("%s"))
         _bar_height_unit = iuPercent;
      else
         _bar_height_unit = iuAbsolute;
   }

   return true;
}

//***************************************************************************
// Search the int parameter
//***************************************************************************

bool cGraphTFTThemeItem::ParseVar(std::string toParse, std::string name, 
                                  int* value)
{
   string val;
   int status;

   if ((status = ParseVar(toParse, name, &val)))
   {
      if (val == "yes" || val == "true")
         *value = 1;
      else if (val == "no" || val == "false")
         *value = 0;
      else
         *value = atoi(val.c_str());
   }

   return status;
}

//***************************************************************************
// Search the string parameter
//***************************************************************************

bool cGraphTFTThemeItem::ParseVar(std::string toParse, std::string name, 
                                  std::string* value)
{
   int posA, posB;

   name += "=";

   if ((posA = toParse.find("," + name)) < 0)
      if ((posA = toParse.find(" " + name)) < 0)
         if ((posA = toParse.find(name)) != 0)
            return false;

   if (posA)
      posA++;

   posB = posA;

   while (true)
   {
      if ((posB = toParse.find(",", posB)) < 0)  
      {
         if ((posB = toParse.find(";", posB)) < 0)
            return false;
      }

      if (!posB || toParse[posB-1] != '\\')
         break;

      posB++;
   }

   *value = toParse.substr(posA + name.size(), posB-posA-name.size());

   if (value->find("\\", 0) >= 0)
   {
      char* buf; char* s; char* d;

      asprintf(&buf, "%s", value->c_str());

      s = d = buf;

      while (*s)
      {
         if (*s != '\\')
            *d++ = *s;

         s++;
      }
      
      *d = 0; *value = buf;
      free(buf);
   }

   return true;
}

//***************************************************************************
// Class cThemeSections
//***************************************************************************
//***************************************************************************
// Get Section By ID
//***************************************************************************

cThemeSection* cThemeSections::getSection(int id)
{
   for (cThemeSection* p = First(); p; p = Next(p)) 
   {
      if (p->getId() == id)
         return p;
   }

   return 0;
}
//***************************************************************************
// Class cThemeSection
//***************************************************************************

string cThemeSection::getPathFromImageMap(const char* name)
{
   char buf[1000];
   char* c = buf;

   strcpy(buf, name);

   // skip leading blanks and numbers

   while (*c && (isdigit(*c) || *c == ' ')) c++;

   string temp = c;
   string temp2 = "";

   d(4, "checking imagemap of section %d", id);

   for (cGraphTFTThemeItem* p = First(); p; p = Next(p)) 
   {
      if (p->Item() == itemMenuImageMap)
      {
         // init translated name at first usage

         if (p->Name() == "")
            p->Name(tr(p->Path().c_str()));
         
         if (p->Name() == temp)
            temp2 = p->Path2();
      }
   }

   if (temp2 != "")
   {
      d(4, "MenuImageMap: Picture for '%s' is '%s'",
        name, temp2.c_str());
   }

   return temp2;
}

//***************************************************************************
// Class cThemeItems
//***************************************************************************
//***************************************************************************
// Get Item By ID
//***************************************************************************

cGraphTFTThemeItem* cThemeItems::getItem(int id)
{
   for (cGraphTFTThemeItem* p = First(); p; p = Next(p)) 
   {
      if (p->Item() == id)
         return p;
   }

   return 0;
}

//***************************************************************************
// Class GraphTFTTheme
//***************************************************************************
//***************************************************************************
// Init
//***************************************************************************

int cGraphTFTTheme::init()
{
   cThemeSection* s = 0;
   cGraphTFTThemeItem* p;

   if (initialized)
      exit();

   // move theme items in their sections

   while ((p = First()))
   {
      // ignore unneeded stuff

      if (!s || p->Section() != s->getId())
         s = getSection(p->Section());

      Del(p, /*delete*/ false);        // remove from list (without deletion)
      s->Add(p);                       // and add to section
   }

   // add items of included sections
   
   for (s = FirstSection(); s; s = NextSection(s))
   {
      cThemeSection* sec;
      cGraphTFTThemeItem* item;

      for (p = s->First(); p; p = s->Next(p)) 
      {
         if (p->Item() == itemSectionInclude && (sec = getSection(p->SectionInclude())))
         {
            cGraphTFTThemeItem* lastItem = 0;
            cGraphTFTThemeItem* newItem;

            for (item = sec->First(); item; item = sec->Next(item))
            {
               // items < itemBegin are not needed
               // item  > itemComplex are to complex, for this we have to copy the
               //         items instead setting the pointer :)

               if (item->Item() >= itemBegin && item->Item() < itemComplex)
               {
                  newItem = new cGraphTFTThemeItem(item);

                  if (!lastItem)
                     s->Ins(newItem, s->First());
                  else
                     s->Add(newItem, lastItem);

                  lastItem = newItem;
               }
            }

            break;
         }
      }
   }

   int x = 0, xs = 0;
   int lastSection = sectionUnknown;

   // init image map, column positions, ...

   for (s = FirstSection(); s; s = NextSection(s)) 
   {
      for (p = s->First(); p; p = s->Next(p)) 
      {
         if (p->Section() != lastSection)
         {
            lastSection = p->Section();
            x = xs = 0;
         }

         // calculate x position of the column items

         if (p->Item() == itemColumn)
         {
            if (p->X() == -1)
               p->X(x);                               // not configured, take calculated
            else if (p->X() < -1 && x + p->X() > 0)
               p->X(x + p->X());                      // go back
            else
               x = p->X();                            // take configured value

            if (p->Width() != -1)
               x += p->Width() + p->Spacing();        // calc next x
         }

         if (p->Item() == itemColumnSelected)
         {
            if (p->X() == -1)
               p->X(xs);                              // not configured, take calculated
            else if (p->X() < -1 && xs + p->X() > 0)
               p->X(xs + p->X());                     // go back
            else
               xs = p->X();                           // take configured value
         
            if (p->Width() != -1)
               xs += p->Width() + p->Spacing();       // calc next x
         }
      }
   }

   initialized = 1;

   return 0;
}

//***************************************************************************
// Exit
//***************************************************************************

int cGraphTFTTheme::exit()
{
   cThemeSection* s;

   for (s = FirstSection(); s; s = NextSection(s))
      s->getItems()->Clear();
   
   sections.Clear();

   // clear of this object's this will well done by 
   // cConfig before new Load() or in destructor!

   initialized = 0;
   
   return 0;
}

//***************************************************************************
// Check
//***************************************************************************

int cGraphTFTTheme::check(const char* theVersion)
{
   // check if the theme fit to current version
      
   if (strcmp(version.c_str(), theVersion) != 0)
   {
      d(1, "Warning: graphtft - themeversion dosen't match");
      return -1;
   }

   return 0;
}

//***************************************************************************
// Load
//***************************************************************************

int cGraphTFTTheme::load(const char* path)
{
   exit();
   Load(path);
   
   return init();
}

//***************************************************************************
// Calss cGraphTFTThemes
//***************************************************************************
//***************************************************************************
// Get Theme
//***************************************************************************

cGraphTFTTheme* cGraphTFTThemes::getTheme(string aTheme)
{
   cGraphTFTTheme* t;

   d(3, "looking for theme '%s'", aTheme.c_str());

   for (t = First(); t; t = Next(t))
      if (t->getName() == aTheme)
         return t;

   return 0;
}
