/**
 *  GraphTFT plugin for the Video Disk Recorder 
 * 
 *  theme.h - A plugin for the Video Disk Recorder
 *
 *  (c) 2004 Lars Tegeler, Sascha Volkenandt  
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 *
 * $Id: theme.h,v 1.2 2006/11/06 10:29:23 root Exp $
 *
 **/

#ifndef __GTFT_THEME_H
#define __GTFT_THEME_H

#include <string>
#include <vdr/config.h>

using std::string;

//***************************************************************************
// Theme Service
//***************************************************************************

class cThemeService
{
   public:

      // items

      enum eThemeItem 
      {
         itemNone,

         itemGeneral,                       // = 1
         itemSectionInclude = itemGeneral,  // = 1
         itemComment,                       // = 2
         itemTheme,

         itemBegin,                         // = 4
         itemText = itemBegin,              // = 4
         itemImage,                         // = 5
         itemImageCover,
         itemImageFile,
         itemDate,
         itemDateOnly,
         itemTimeOnly,                      // = 10
         itemChannel,
         itemChannelOnly,
         itemChannelNumber,
         itemRectangle,
         itemLogo,
         itemTimebar,
         itemPresentTitle,
         itemPresentSubtitle,
         itemPresentTime,
         itemFollowingTitle,                // = 20
         itemFollowingSubtitle,
         itemFollowingTime,
         itemRecording,
         itemSymRecording,
         itemSym2ch,
         itemSymDD,
         itemSymVTX,
         itemSymCrypt,
         itemSymRepPlay,
         itemSymRepPause,                   // = 30
         itemSymRepFF,
         itemSymRepFB,
         itemSymRepSF,
         itemSymRepSB,
         itemMessage,
         itemMessageBackground,
         itemVolumeMuteSymbol,
         itemVolumebar,
         itemVolumeBackground,
         itemReplayName,                    // = 40
         itemReplayFolder,
         itemReplayProcessbar,
         itemReplayTotal,
         itemReplayCurrent,
         itemReplayRemaining,
         itemMenuTitle,
         itemMenu,
         itemMenuSelected,
         itemMenuStaticPicture,
         itemMenuStaticX,                   // = 50
         itemMenuStaticY,
         itemMenuButtonRed,
         itemMenuButtonGreen,
         itemMenuButtonBlue,
         itemMenuButtonYellow,
         itemMenuButtonBackgroundRed,
         itemMenuButtonBackgroundGreen,
         itemMenuButtonBackgroundBlue,
         itemMenuButtonBackgroundYellow,
         itemMenuText,                       // = 60
         itemMenuTextBackground,
         itemMenuImageMap,
         itemSpectrumAnalyzer,
         itemPartingLine,                    // = 64
         itemDateTimeFormat,
         itemClickArea,
         itemMenuNavigationArea,
         itemCalibrationCursor,

         itemComplex,
         itemColumn = itemComplex,
         itemColumnSelected,

         itemEnd
      };

      // sections

      enum eThemeSection
      {
         sectionUnknown          = -1,
         sectionTheme,

         sectionNormalTV,
         sectionNormalRadio,
         sectionNormalClock,
         sectionNormalLogo,
	 sectionNormalSysinfo,
         sectionNormalSoccer,

         sectionReplayNormal,
         sectionReplayDVD,
         sectionReplayMP3,

         sectionMenuCommon,
         sectionMenu,
         sectionMenuWhatsOnElse,
         sectionMenuWhatsOn,
         sectionMenuWhatsOnNow,
         sectionMenuWhatsOnNext,
         sectionMenuMain,
         sectionMenuSchedule,
         sectionMenuChannels,
         sectionMenuTimers,
         sectionMenuRecordings,
         sectionMenuSetup,
         sectionMenuSetupPage,
         sectionMenuCommands,

         sectionCalibration,

         sectionCount
      };

      enum ItemUnit
      {
         iuUnknown = -1,
         iuAbsolute,
         iuRelative,
         iuPercent
      };

      static const char* toName(eThemeSection aSection);
      static eThemeSection toSection(const char* aName);
      static int isValid(eThemeSection aSection) 
         { return aSection > sectionUnknown && aSection < sectionCount; }

      static const char* sections[sectionCount+1];
};

//***************************************************************************
// Class cGraphTFTThemeItem
//  - represents a theme item
//***************************************************************************

class cGraphTFTThemeItem : public cListObject, cThemeService
{  
   private:   

      int _item, _section, _sectionInclude;

      int _x,_y,_width,_height,_red,_green,_blue,_transparent,_lines,_size,_sw;
      int _bg_x,_bg_y,_bg_width,_bg_heigth;
      int _bg_red,_bg_green,_bg_blue,_bg_transparent;
      int _stat_pic,_stat_x,_stat_y, _image_map, _stat_text, _stat_width;
      int _align, _animated, _delay, _count, _number;
      int _index, _spacing, _bar_height, _bar_height_unit;

      string _onClick;
      string _onDblClick;
      string _font;
      string _name;
      string _path;
      string _path2;
      string _type;
      string _format;
      string _text;
     
   protected:

      bool ParseImage(string toParse);
      bool ParseText(string toParse);
  
      bool ParseVar(string toParse, string name, int* value);
      bool ParseVar(string toParse, string name, string *value);

   public:

      cGraphTFTThemeItem(void);
      cGraphTFTThemeItem(cGraphTFTThemeItem* item);
      ~cGraphTFTThemeItem();
      
      bool Parse(const char *s);
      int Item(void)               { return _item; }
      int Section(void)            { return _section; }

      int Index(int value = -1)          { if (value != -1) _index = value; return _index; }
      int X(int value = -1)              { if (value != -1) _x = value; return _x; }
      string Name(const char* value = 0) { if (value) _name = value; return _name; }

      int BgX(void)            const { return _bg_x; }
      int Y(void)              const { return _y; }
      int BgY(void)            const { return _bg_y; }
      int ImageMap(void)       const { return _image_map; }
      int StaticPicture(void)  const { return _stat_pic; }
      int StaticText(void)     const { return _stat_text; }
      int StaticWidth(void)    const { return _stat_width; }
      int StaticX(void)        const { return _stat_x; }
      int StaticY(void)        const { return _stat_y; }
      int Width(void)          const { return _width; }
      int BgWidth(void)        const { return _bg_width; }
      int Height(void)         const { return _height; }
      int BgHeight(void)       const { return _bg_heigth; }
      int Red(void)            const { return _red; }
      int BgRed(void)          const { return _bg_red; }
      int Green(void)          const { return _green; }
      int BgGreen(void)        const { return _bg_green; }
      int Blue(void)           const { return _blue; }
      int BgBlue(void)         const { return _bg_blue; }
      int Transparent(void)    const { return _transparent; }
      int BgTransparent(void)  const { return _bg_transparent; }
      int Lines(void)          const { return _lines; }
      int Size(void)           const { return _size; }  
      int Switch(void)         const { return _sw; }
      int Align(void)          const { return _align; }
      int Animated(void)       const { return _animated; }
      int Delay(void)          const { return _delay; }
      int Count(void)          const { return _count; }
      int Number(void)         const { return _number; }
      int Spacing(void)        const { return _spacing; }
      int BarHeight(void)      const { return _bar_height; }
      int BarHeightUnit(void)  const { return _bar_height_unit; }
      string Font(void)        const { return _font; }
      string Path(void)        const { return _path; }
      string Path2(void)       const { return _path2; }
      string Type(void)        const { return _type; }
      string Format()          const { return _format; }
      string Text()            const { return _text; }
      string OnClick()         const { return _onClick; }
      string OnDblClick()      const { return _onDblClick; }

      int SectionInclude(void) const { return _sectionInclude; }
};

//***************************************************************************
// Class cThemeItems
//    - holding items of one section
//***************************************************************************

class cThemeItems : public cList<cGraphTFTThemeItem>
{
   public:

      cGraphTFTThemeItem* getItem(int id);

   protected:

};

//***************************************************************************
// Class cThemeSection
//  - represents a section of a theme
//***************************************************************************

class cThemeSection : public cListObject, cThemeService
{
   public:

      cThemeSection(int aId)                   { id = aId; }

      int getId()                              { return id; }
      cThemeItems* getItems()                  { return &items; }

      cGraphTFTThemeItem* getItem(int itemId)  { return items.getItem(itemId); }

      cGraphTFTThemeItem* First()              { return items.First(); }
      cGraphTFTThemeItem* Next(cGraphTFTThemeItem* item)  
      { return items.Next(item); }

      void Ins(cGraphTFTThemeItem* item, cGraphTFTThemeItem* before = 0) 
      { items.Ins(item, before); }

      void Add(cGraphTFTThemeItem* item, cGraphTFTThemeItem* after = 0)  
      { items.Add(item, after); }

      string getPathFromImageMap(const char* name);

   protected:

      int id;
      cThemeItems items;
};

//***************************************************************************
// Class cThemeSections
//    - holding all sections
//***************************************************************************

class cThemeSections : public cList<cThemeSection>
{
   public:

      cThemeSection* getSection(int id);

   protected:

};

//***************************************************************************
// Class cGraphTFTTheme
//   - holding all Items includes in the thme configuration file
//***************************************************************************

class cGraphTFTTheme : public cConfig<cGraphTFTThemeItem>, public cListObject, cThemeService
{
   public:

      cGraphTFTTheme()    { initialized = 0; }
      ~cGraphTFTTheme()   { exit(); }

      // functions

      int init();
      int exit();
      int check(const char* theVersion);
      int load(const char* path);

      // get

      string getName()                    { return name; }
      string getVersion()                 { return version; }
      string getDir()                     { return dir; }
      string getStartImage()              { return startImage; }
      string getEndImage()                { return endImage; }

      // set

      void setName(string aValue)         { name = aValue; }
      void setVersion(string aValue)      { version = aValue; }
      void setDir(string aValue)          { dir = aValue; }
      void setStartImage(string aValue)   { startImage = aValue; }
      void setEndImage(string aValue)     { endImage = aValue; }

      // the sections

      cThemeSection*  getSection(int id)  { return sections.getSection(id); }
      cThemeSections* getSections()       { return &sections; }

      cThemeSection* FirstSection()              
      { return sections.First(); }

      cThemeSection* NextSection(cThemeSection* sect)
      { return sections.Next(sect); }


   protected:

      // data

      string dir;
      string name;
      string version;
      string startImage;
      string endImage;
      int initialized;
      cThemeSections sections;
};

//***************************************************************************
// Class cGraphTFTThemes
//    - holding all themes
//***************************************************************************

class cGraphTFTThemes : public cList<cGraphTFTTheme>
{
   public:

      cGraphTFTTheme* getTheme(string aTheme);

   protected:

};

//***************************************************************************
// External
//***************************************************************************

extern cGraphTFTTheme* theTheme;
extern cGraphTFTThemes themes;

//***************************************************************************
#endif //__GTFT_THEME_H
